/*==============================================================================


   Purpose:    determine MHz rate the processor is running


   Author:     Rudolf Berrendorf
               University of Applied Sciences Bonn-Rhein-Sieg
               Computer Science Department
               53754 Sankt Augustin
               rudolf.berrendorf@fh-bonn-rhein-sieg.de


   Date:       25.09.2001


   DISCLAIMER

                  PCL - The Performance Counter Library


   Copyright (C) 1999, 2000
   Forschungszentrum Juelich GmbH, Federal Republic of Germany.
   Copyright (C) 2001
   University of Applied Sciences Bonn-Rhein-Sieg

   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

     - Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

     - Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

     - Any publications that result from the use of this software shall
       reasonably refer to the Research Centre's development.

     - All advertising materials mentioning features or use of this software
       must display the following acknowledgement:

           This product includes software developed by Forschungszentrum
           Juelich GmbH, Federal Republic of Germany.

     - Forschungszentrum Juelich GmbH is not obligated to provide the user with
       any support, consulting, training or assistance of any kind with regard
       to the use, operation and performance of this software or to provide
       the user with any updates, revisions or new versions.


   THIS SOFTWARE IS PROVIDED BY FORSCHUNGSZENTRUM JUELICH GMBH "AS IS" AND ANY
   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL FORSCHUNGSZENTRUM JUELICH GMBH BE LIABLE FOR
   ANY SPECIAL, DIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
   RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
   CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
   CONNECTION WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.

==============================================================================*/


#include "../../determine_mhz.h"

#include <stdio.h>
#include <stdlib.h>


/*                                                                            */
/*============================================================================*/
/* returns MHz-rate of the processor (or -1 if not available)                 */


int
PCL_determine_mhz_rate(void)

{
  int mhz;
  float fp_mhz;
  char *s, line[256], cmd[80];
  FILE *f;
  char cmd_line[80], fname[L_tmpnam];

  
  /*??? system call takes very long */
  sprintf(cmd_line, "fpversion >%s", tmpnam(fname));
  if(system(cmd_line) == -1)
    {
      remove(fname);
      return -1;
    }

  /* open output file */
  if((f = fopen(fname, "r")) == NULL)
    {
      remove(fname);
      return -1;
    }

  /* ignore all lines until we reach something with a CPU's clock rate line */
  while((s = fgets(line, 256, f)) != NULL)
    {
      if(strstr(line, "CPU's clock rate is"))
	break;
    }

  /* remove temporary file */
  remove(fname);

  if(s == NULL)
    /* for some reason nothing found */
    mhz = -1;
  else
    {
      if(sscanf(line, "%s %s %s %s %s %s %f",
		cmd, cmd, cmd, cmd, cmd, cmd, &fp_mhz) != 7)
	mhz = -1;
      else
	mhz = (int)fp_mhz;
    }

  return mhz;
}


/*============================================================================*
 *                             that's all folks                               *
 *============================================================================*/
