/*==============================================================================


   Purpose:    small test program for hpm


   Author:     Rudolf Berrendorf
               University of Applied Sciences Bonn-Rhein-Sieg
               Computer Science Department
               53754 Sankt Augustin
               rudolf.berrendorf@fh-bonn-rhein-sieg.de


   last mod:   27.01.2002


==============================================================================*/

/* This is a test program for hpm. Execute YOUR_BINARY_PATH/hpm a.out after
   compiling this program.
 */


#include <stdio.h>

#include "../src/processor_number.h"

/* dimension of test arrays */
#define N 100

#define ITER_COUNT 100

/*                                                                            */
/*============================================================================*/
/* matrix addition as a simple test program.                                  */


static void
matadd(double a[N][N], double b[N][N], double c[N][N], int n)

{
  int i, j;


  for (i = 0; i < n; ++i)
    for (j = 0; j < n; ++j)
      a[i][j] = b[i][j] + c[i][j];
}

/*============================================================================*/

int main(int argc, char **argv)
{
  double a[N][N], b[N][N], c[N][N];
  int i,j,n=N;

  printf("\nProgram to be used for PCL measurement tests of sub-processes\n"
	 "Starting %s on processor %d\n", argv[0], PCL_processor_number());

  if(argc > 1)
    {
      if((sscanf(argv[1], "%d", &n) != 1)
	 || (n > N))
	printf("wrong argument (%s)\n", argv[1]);
    }

  /* initialize test arrays */
  for (i = 0; i < n; ++i)
    for (j = 0; j < n; ++j)
      a[i][j] = b[i][j] = c[i][j] = 1.0;

  /* do some initial work */
  for(i = 0; i < ITER_COUNT; ++i)
    matadd(a, b, c, n);
  
  printf("stopping %s (should be roughly %d FP operations)\n",
	 argv[0], n*n*ITER_COUNT);

  return 0;
}

/*============================================================================*/
