/*==============================================================================


   Purpose:    test PCL time functions


   Author:     Rudolf Berrendorf
               University of Applied Sciences Bonn-Rhein-Sieg
               Computer Science Department
               53754 Sankt Augustin
               rudolf.berrendorf@fh-bonn-rhein-sieg.de


   last mod:   05.04.2000


   DISCLAIMER

                  PCL - The Performance Counter Library


   Copyright (C) 2000
   Forschungszentrum Juelich GmbH, Federal Republic of Germany.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

     - Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

     - Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

     - Any publications that result from the use of this software shall
       reasonably refer to the Research Centre's development.

     - All advertising materials mentioning features or use of this software
       must display the following acknowledgement:

           This product includes software developed by Forschungszentrum
           Juelich GmbH, Federal Republic of Germany.

     - Forschungszentrum Juelich GmbH is not obligated to provide the user with
       any support, consulting, training or assistance of any kind with regard
       to the use, operation and performance of this software or to provide
       the user with any updates, revisions or new versions.


   THIS SOFTWARE IS PROVIDED BY FORSCHUNGSZENTRUM JUELICH GMBH "AS IS" AND ANY
   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL FORSCHUNGSZENTRUM JUELICH GMBH BE LIABLE FOR
   ANY SPECIAL, DIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
   RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
   CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
   CONNECTION WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
  
==============================================================================*/

#include <stdio.h>

#include "pcl.h"


/*                                                                            */
/*============================================================================*/
/* test program:                                                              */


int main(int argc, char **argv)
{
  PCL_TIME_TYPE e0, e1, e2;
  double e_over;
  int count;

  printf("\nProgram to test time functionality in PCL\n");

  /* init calls */
  e0 = PCLesecond();


  /*--------------------------------------------------------------------------*/
  /* measure overhead */
  e0 = PCLesecond();
  count = 0;
  while ((e1 = PCLesecond() - e0) < 1)
    ++count;
  e_over = 1.0 / count;
  printf("overhead   PCLesecond : %12.9f\n", e_over);


/*--------------------------------------------------------------------------*/

  /* measure resolution */
  for (e0 = PCLesecond(), e1 = PCLesecond(), e2 = PCLesecond();
       (e1 == e0) || (e2 == e1);
       e1 = PCLesecond(), e2 = PCLesecond())
    ;
  printf("resolution PCLesecond < %12.9f\n", e2 - e0);

/*--------------------------------------------------------------------------*/

  printf("clock test (5 seconds)\a:");
  fflush(stdout);
  e0 = PCLesecond();
  while ((e1 = PCLesecond() - e0) < 5)
    ;
  printf("\a %12.9f\n", e1);

  return 0;
}


/*============================================================================*
 *                             that's all folks                               *
 *============================================================================*/
