#include <stdio.h>
#include <fcntl.h>
#include <math.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/processor.h>
#include <sys/procset.h>
#include "perfmon.h"

#define SIZE 150
static double A[SIZE][SIZE];
static double C[SIZE][SIZE];
static double B[SIZE][SIZE];
void matrix()
{
    int i, j, k;
    double tmp;

    for (i = 0; i < SIZE; i++)
	for (j = 0; j < SIZE; j++) {
	    tmp = 0.0;
	    for (k = 0; k < SIZE; k++)
		tmp += A[i][k] * B[k][j];
	    C[i][j] = tmp;
	}
}

main(int argc, char *argv[])
{
    int fd;
    int rc;
    int cpu = 0;
    char c;
    extern char *optarg;
    extern int optind;
    register unsigned long long a, b;
    register unsigned long x, y;
    hrtime_t start, end;
    unsigned long long tmp;

    /* Parse command line arguments */
    while ((c = getopt(argc, argv, "p:")) != EOF)
	switch(c) {
	    /* -p cpu : binds this process to the indicated CPU */
	    case 'p':
		cpu = atoi(optarg);
		break;

	    default:
		break;
	}

    /* Try to bind us to the given CPU */
    rc = processor_bind(P_PID, P_MYID, cpu, NULL);
    if (rc == -1) {
	perror("processor_bind()");
	exit(1);
    }

    fd = open("/dev/perfmon", O_RDONLY);
    if (fd == -1) {
	perror("open(/dev/perfmon)");
	exit(1);
    }

    /* Set PCR to accumulate cycle count and number of instructions */
    tmp = PCR_S0_CYCLE_CNT | PCR_S1_INSTR_CNT | PCR_USER_TRACE;
    rc = ioctl(fd, PERFMON_SETPCR, &tmp);
    if (rc < 0) {
	perror("ioctl(PERFMON_SETPCR)");
	exit(1);
    }

    /* Flush all caches */
    ioctl(fd, PERFMON_FLUSH_CACHE);

    /* Run Loop */
    printf("Sqare matrix dimensions: %d x %d\n", SIZE, SIZE);
    printf("Storage required for 1 matrix: %d KB\n",
	(sizeof(double)*SIZE*SIZE)/1024);
    printf("Running cold matrix multiply...");
    fflush(stdout);
    start = gethrtime();
    clr_pic();
    cpu_sync();
    a = get_pic();
    matrix();
    cpu_sync();
    b = get_pic();
    end = gethrtime();
    printf("Done.\n\n");

    /* Calculate cold CPI */
    x = extract_pic0(b) - extract_pic0(a);  /* Number of cycles */
    y = extract_pic1(b) - extract_pic1(a);  /* Number of instructions */
    printf("Wall clock time used:            %.1lf msec\n",
	(end - start) / (double)1000000);
    printf("Number of instructions executed: %lu\n", y);
    printf("Number of cycles taken:          %lu\n", x);
    printf("CPI for cold multiply:           %.4lf\n", (double)x/(double)y);
    printf("IPC for cold multiply:           %.4lf\n", (double)y/(double)x);

    /* Clean up */
    close(fd);
}
